T(MxBase_Alloc)
T(MxBase_FreeData)
T(MxBase_AllocData)
T(SortSeqIndexes)
T(Alloc_Vectors)
T(MainLoop_NotNW)
T(WriteOutput)
T(NWB)
T(ReadAllStdioFile)
T(Windex_Init)
T(Windex_SetSeqIndex)
T(SeqToWords)
T(SeqToWordsStep)
T(SeqToShortWords)
T(SeqToShortWordsA)
T(SeqToShortWordsB)
T(GetFractIdB)
T(Windex_UniqueWordsAlloc)
T(Windex_UniqueWords)
T(GetPctId)
T(Windex_Reset)
T(GetSig)
T(NWEditDist)
T(EditDist_Myers)
T(EditDist_BlockTarget)
T(NWBand)
T(WordCounting)
T(NWAff)
T(NWAffBand)
T(NWSimple)
T(NWSimpleB)
T(BandWrap)
T(IncIdCounts)
T(GetBestDiagB)
T(GetBestDiagB1)
T(GetBestDiagB2)
T(ClusterInit)
T(ClusterPrep)
T(HotSort1)
T(HotSort2)
T(SortA)
T(SortB)
T(CountSort)
T(AddWords)
T(ClusterWindex)
T(MainInit)
T(Output)
T(WindexTail)
T(WindexExit)
T(Sort)
T(U_AllocSeqLength)
T(U_AllocSeedCount)
T(U_AddSeed)
T(AddSeq)
T(U_SetWordCounts)
T(U_SetWordCountsHash)
T(U_SetWordScores)
T(U_SetHotHits)
T(U_SetHotHitsHash)
T(U_SetHotHitsScores)
T(U_Search)
T(U_SearchExact)
T(WF_SeqToWords)
T(WF_SeqToWordsA)
T(WF_SeqToWordsB)
T(WF_AllocLA)
T(WF_AllocLB)
T(WF_AllocDiags)
T(WF_SetA)
T(WF_SetA_Nb)
T(WF_SetAZero)
T(WF_SetA2)
T(WF_SetB)
T(WF_GetCommonWordCount)
T(WF_GetBestDiag)
T(GetFractIdGivenPath)
T(WX_GetUniqueWords)
T(CompressPath)
T(GetHSPs1)
T(GetHSPs2)
T(AlignHSPs)
T(WF_ResolveHSPs)
T(WX_SetExcludes)
T(ViterbiFast)
T(ViterbiFastBand)
T(ViterbiFastBand0)
T(ViterbiFastBand1)
T(ViterbiFastBand2)
T(ViterbiFastBand3)
T(ViterbiFastBand4)
T(TraceBackBit)
T(TraceBackBitSW)
T(SF_GetNextSeq)
T(SF_FillCache)
T(OnGlobalAccept)
T(UngappedBlast)
T(UngappedBlastId)
T(UngappedBlast2Hit)
T(LogHSPs)
T(BlastOutput)
T(BlastLeft)
T(BlastRight)
T(Blast1)
T(Blast2)
T(Blast3)
T(Blast4)
T(GetBestSeg)
T(SWLinearDP)
T(SWLinearTB)
T(SWLinearDP2)
T(SWLinearTB2)
T(Chain)
T(XlatSeq)
T(XlatSeqToLetters)
T(XDropFwdSimple)
T(XDropFwdFast)
T(XDropFwdFastTB)
T(XDropBwd)
T(SWSimple)
T(PathAlloc)
T(SubPath)
T(SWUngapped)
T(SWFast)
T(SWFastNTB)
T(SWAT_CacheQuery)
T(SWAT_AlignTarget)
T(SWAT_CacheQueryNW)
T(SWAT_AlignTargetNW)
T(SeqDB_FromFasta)
T(LocalUngappedHitToAD)
T(LocalGappedHitToAD)
T(GlobalHitToAD)
T(ResolveOverlaps)
T(GetORFs)
T(ChainCov_AddHit)
T(ChainCov_EndQuery)
T(ChainCov_DoTarget)
T(BuildNb)
T(MakeIntSubstMx)
T(UngappedExtendLeft)
T(UngappedExtendRight)
T(AlignSP)
T(AlignHSP)

// Background
T(Bg_SearchLoop)
T(Bg_MainInit)
T(Bg_MainTerm)
T(Bg_Other)
T(Bg_1)
T(Bg_2)
T(Bg_3)
T(Bg_4)
T(Bg_5)
T(Bg_6)
T(Bg_7)
T(Bg_8)
T(Bg_9)
T(Bg_XFrame2)
T(Bg_Usearch1)
T(Bg_Usearch2)
T(Bg_Usearch3)
T(Bg_Usearch4)
T(Bg_Hot)

// For Timer2
T(Search_2)
T(Search_Loop_2)
T(Search_InnerLoop_2)
T(OnHit_2)
T(UngappedBlast_2)
T(MainInit_2)
T(MainTerm_2)
