//
//  fakeoptimatrix.hpp
//  Mothur
//
//  Created by Sarah Westcott on 4/20/17.
//  Copyright © 2017 Schloss Lab. All rights reserved.
//

#ifndef fakeoptimatrix_hpp
#define fakeoptimatrix_hpp

#include "optidata.hpp"

class FakeOptiMatrix : public OptiData {
    
public:
    FakeOptiMatrix(); 
    ~FakeOptiMatrix(){ }
};


#endif /* fakeoptimatrix_hpp */
