"Path: .\n"
"URL: file:///public/svn/usearch\n"
"Repository Root: file:///public/svn/usearch\n"
"Repository UUID: 58640331-1837-4c17-bc3e-636dc59aced1\n"
"Revision: 34\n"
"Node Kind: directory\n"
"Schedule: normal\n"
"Last Changed Author: bob\n"
"Last Changed Rev: 34\n"
"Last Changed Date: 2011-05-01 08:29:04 -0700 (Sun, 01 May 2011)\n"
"\n"
"?       mk\n"
"!       svnmods.h\n"
"M       ungappedblastid.cpp\n"
"M       chaindisjointhits.cpp\n"
